/* Check whether a section type is a valid target for relocation.
   Copyright (C) 2014, 2015 Red Hat, Inc.
   This file is part of elfutils.

   This file is free software; you can redistribute it and/or modify
   it under the terms of either

     * the GNU Lesser General Public License as published by the Free
       Software Foundation; either version 3 of the License, or (at
       your option) any later version

   or

     * the GNU General Public License as published by the Free
       Software Foundation; either version 2 of the License, or (at
       your option) any later version

   or both in parallel, as here.

   elfutils is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received copies of the GNU General Public License and
   the GNU Lesser General Public License along with this program.  If
   not, see <http://www.gnu.org/licenses/>.  */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <libeblP.h>


bool
ebl_check_reloc_target_type (Ebl *ebl, Elf64_Word sh_type)
{
  if (ebl->check_reloc_target_type (ebl, sh_type))
    return true;

  switch (sh_type)
    {
      case SHT_PROGBITS:
      case SHT_NOBITS:
      case SHT_INIT_ARRAY:
      case SHT_FINI_ARRAY:
      case SHT_PREINIT_ARRAY:
      case SHT_NOTE:
	return true;

      default:
	return false;
    }
}
